@@ -34,40 +34,56 @@ SUPPORTED_VERSIONS = [
     '1.16',
     '1.17',
     '1.18',
+    '1.19',
+    '1.20',  # unused
+    '1.21',
+    '1.22',
 ]
 
 
-def _op(func, b):
-    return lambda a: func(StrictVersion(a), StrictVersion(b))
+def _op(func, b, msg):
+    return lambda a: func(StrictVersion(a), StrictVersion(b)) or msg
 
 
 def lt(b):
-    return _op(operator.lt, b)
+    msg = 'requires version less than %s' % b
+    return _op(operator.lt, b, msg)
 
 
 def le(b):
-    return _op(operator.le, b)
+    msg = 'requires at most version %s' % b
+    return _op(operator.le, b, msg)
 
 
 def eq(b):
-    return _op(operator.eq, b)
+    msg = 'requires version %s' % b
+    return _op(operator.eq, b, msg)
 
 
 def ne(b):
-    return _op(operator.ne, b)
+    msg = 'can not use version %s' % b
+    return _op(operator.ne, b, msg)
 
 
 def ge(b):
-    return _op(operator.ge, b)
+    msg = 'requires at least version %s' % b
+    return _op(operator.ge, b, msg)
 
 
 def gt(b):
-    return _op(operator.gt, b)
+    msg = 'requires version greater than %s' % b
+    return _op(operator.gt, b, msg)
 
 
 def _compare(ver, *predicates, **kwargs):
     func = kwargs.get('op', all)
-    return func(p(ver) for p in predicates)
+    if func(p(ver) is True for p in predicates):
+        return True
+    # construct an error message if the requirement not satisfied
+    err_msg = 'Operation or argument is not supported with version %s; ' % ver
+    err_detail = [p(ver) for p in predicates if p(ver) is not True]
+    logic = ', and ' if func is all else ', or '
+    return err_msg + logic.join(err_detail)
 
 
 def compare(ver, *predicates, **kwargs):
@@ -82,10 +98,10 @@ def compare(ver, *predicates, **kwargs):
 
     """
     exc = kwargs.get('exc', True)
-    if not _compare(ver, *predicates, **kwargs):
+    result = _compare(ver, *predicates, **kwargs)
+    if result is not True:
         if exc:
-            raise ValueError(
-                'Operation or argument is not supported with version %s' % ver)
+            raise ValueError(result)
         return False
     return True
 
